searchData={"items":[{"type":"module","doc":"Wasmex is a fast and secure [WebAssembly](https://webassembly.org/) and [WASI](https://github.com/WebAssembly/WASI) runtime for Elixir.\nIt enables lightweight WebAssembly containers to be run in your Elixir backend.\nThis is the main module, providing most of the needed API to run Wasm binaries.\n\nIt uses [wasmtime](https://wasmtime.dev) to execute Wasm binaries through a [Rust](https://www.rust-lang.org) NIF.\n\nEach Wasm module must be compiled from a `.wasm` or '.wat' file.\nA compiled Wasm module can be instantiated which usually happens in a [GenServer](https://hexdocs.pm/elixir/master/GenServer.html).\nTo start the GenServer, `start_link/1` is used:\n\n    iex> bytes = File.read!(TestHelper.wasm_test_file_path())\n    iex> {:ok, instance_pid} = Wasmex.start_link(%{bytes: bytes})\n    iex> Wasmex.call_function(instance_pid, \"sum\", [50, -8])\n    {:ok, [42]}\n\nMemory of a Wasm instance can be read/written using `Wasmex.Memory`:\n\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasm_test_file_path())})\n    iex> {:ok, store} = Wasmex.store(pid)\n    iex> {:ok, memory} = Wasmex.memory(pid)\n    iex> index = 4\n    iex> Wasmex.Memory.set_byte(store, memory, index, 42)\n    iex> Wasmex.Memory.get_byte(store, memory, index)\n    42\n\nSee `start_link/1` for starting and configuring a Wasm instance and `call_function/3` for details about calling Wasm functions.","title":"Wasmex","ref":"Wasmex.html"},{"type":"function","doc":"Calls a function with the given `name` and `params` on the Wasm instance\nand returns its results.","title":"Wasmex.call_function/4","ref":"Wasmex.html#call_function/4"},{"type":"function","doc":"iex> wat = \"(module\n    ...>          (func $helloWorld (result i32) (i32.const 42))\n    ...>          (export \\\"hello_world\\\" (func $helloWorld))\n    ...>        )\"\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: wat})\n    iex> Wasmex.call_function(pid, \"hello_world\", [])\n    {:ok, [42]}","title":"Example - Wasmex.call_function/4","ref":"Wasmex.html#call_function/4-example"},{"type":"function","doc":"Strings are common candidates for function parameters and return values.\nHowever, they can not be used directly when calling Wasm functions,\nbecause Wasm only knows number data types.\nSince Strings are just \"a bunch of bytes\", we can write these bytes into memory\nand give our Wasm function a pointer to that memory location.","title":"String Handling - Wasmex.call_function/4","ref":"Wasmex.html#call_function/4-string-handling"},{"type":"function","doc":"Given we have the following Rust function that returns the first byte of a string input\ncompiled to Wasm:\n\n```rust\n#[no_mangle]\npub extern \"C\" fn string_first_byte(bytes: *const u8, length: usize) -> u8 {\n    let slice = unsafe { slice::from_raw_parts(bytes, length) };\n    match slice.first() {\n        Some(&i) => i,\n        None => 0,\n    }\n}\n```\n\nThis Wasm function can be called from Elixir:\n\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasm_test_file_path())})\n    iex> {:ok, store} = Wasmex.store(pid)\n    iex> {:ok, memory} = Wasmex.memory(pid)\n    iex> index = 42\n    iex> string = \"hello, world\"\n    iex> Wasmex.Memory.write_binary(store, memory, index, string)\n    iex> Wasmex.call_function(pid, \"string_first_byte\", [index, String.length(string)])\n    {:ok, [104]} # 104 is the letter \"h\" in ASCII/UTF-8 encoding\n\nPlease note that Elixir and Rust assume Strings to be valid UTF-8. Take care when handling other encodings.","title":"Strings as Function Parameters - Wasmex.call_function/4","ref":"Wasmex.html#call_function/4-strings-as-function-parameters"},{"type":"function","doc":"Given we have the following Rust function compiled to Wasm (again, copied from our test code):\n\n```rust\n#[no_mangle]\npub extern \"C\" fn string() -> *const u8 {\n    b\"Hello, World!\".as_ptr()\n}\n```\n\nThis function returns a _pointer_ to its memory.\nThis memory location contains the String \"Hello, World!\".\n\nThis is how we would receive this String in Elixir:\n\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasm_test_file_path())})\n    iex> {:ok, store} = Wasmex.store(pid)\n    iex> {:ok, memory} = Wasmex.memory(pid)\n    iex> {:ok, [pointer]} = Wasmex.call_function(pid, \"string\", [])\n    iex> Wasmex.Memory.read_string(store, memory, pointer, 13)\n    \"Hello, World!\"","title":"Strings as Function Return Values - Wasmex.call_function/4","ref":"Wasmex.html#call_function/4-strings-as-function-return-values"},{"type":"function","doc":"The default timeout for `call_function` is 5 seconds, or 5000 milliseconds.\nWhen calling a long-running function, you can specify a timeout value (in milliseconds) for this call.\n\n    iex> wat = \"(module\n    ...>          (func $helloWorld (result i32) (i32.const 42))\n    ...>          (export \\\"hello_world\\\" (func $helloWorld))\n    ...>        )\"\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: wat})\n    iex> Wasmex.call_function(pid, \"hello_world\", [], 10_000)\n    {:ok, [42]}\n\nIn the example above, we specify a timeout of 10 seconds.","title":"Specifying a timeout - Wasmex.call_function/4","ref":"Wasmex.html#call_function/4-specifying-a-timeout"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Wasmex.child_spec/1","ref":"Wasmex.html#child_spec/1"},{"type":"function","doc":"Returns whether a function export with the given `name` exists in the Wasm instance.","title":"Wasmex.function_exists/2","ref":"Wasmex.html#function_exists/2"},{"type":"function","doc":"iex> wat = \"(module\n    ...>          (func $helloWorld (result i32) (i32.const 42))\n    ...>          (export \\\"hello_world\\\" (func $helloWorld))\n    ...>        )\"\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: wat})\n    iex> Wasmex.function_exists(pid, \"hello_world\")\n    true\n    iex> Wasmex.function_exists(pid, \"something_else\")\n    false","title":"Examples - Wasmex.function_exists/2","ref":"Wasmex.html#function_exists/2-examples"},{"type":"function","doc":"Returns the `Wasmex.Instance` of the Wasm instance.","title":"Wasmex.instance/1","ref":"Wasmex.html#instance/1"},{"type":"function","doc":"iex> {:ok, pid} = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasm_test_file_path())})\n    iex> {:ok, %Wasmex.Instance{}} = Wasmex.instance(pid)","title":"Example - Wasmex.instance/1","ref":"Wasmex.html#instance/1-example"},{"type":"function","doc":"Returns the exported `Wasmex.Memory` of the given Wasm instance.","title":"Wasmex.memory/1","ref":"Wasmex.html#memory/1"},{"type":"function","doc":"iex> {:ok, pid} = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasm_test_file_path())})\n    iex> {:ok, %Wasmex.Memory{}} = Wasmex.memory(pid)","title":"Example - Wasmex.memory/1","ref":"Wasmex.html#memory/1-example"},{"type":"function","doc":"Returns the `Wasmex.Module` of the Wasm instance.","title":"Wasmex.module/1","ref":"Wasmex.html#module/1"},{"type":"function","doc":"iex> {:ok, pid} = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasm_test_file_path())})\n    iex> {:ok, %Wasmex.Module{}} = Wasmex.module(pid)","title":"Example - Wasmex.module/1","ref":"Wasmex.html#module/1-example"},{"type":"function","doc":"Starts a GenServer which compiles and instantiates a Wasm module from the given `.wasm` or `.wat` bytes.\n\n    iex> bytes = File.read!(TestHelper.wasm_test_file_path())\n    iex> {:ok, _pid} = Wasmex.start_link(%{bytes: bytes})\n\nAlternatively, a precompiled `Wasmex.Module` can be passed with its `Wasmex.Store`:\n\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, module} = Wasmex.Module.compile(store, \"(module)\")\n    iex> {:ok, _pid} = Wasmex.start_link(%{store: store, module: module})","title":"Wasmex.start_link/1","ref":"Wasmex.html#start_link/1"},{"type":"function","doc":"Wasm imports may be given as an additional option.\nImports are a map of namespace-name to namespaces.\nEach namespace is in turn a map of import-name to import.\n\n    iex> wat = \"(module\n    ...>          (import \\\"IO\\\" \\\"inspect\\\" (func $log (param i32)))\n    ...>        )\"\n    iex> io_inspect = fn (%{_memory: %Wasmex.Memory{}, _caller: %Wasmex.StoreOrCaller{}} = _context, i) ->\n    ...>                IO.inspect(i)\n    ...>              end\n    iex> imports = %{\n    ...>   IO: %{\n    ...>     inspect: {:fn, [:i32], [], io_inspect},\n    ...>   }\n    ...> }\n    iex> {:ok, _pid} = Wasmex.start_link(%{bytes: wat, imports: imports})\n\nIn the example above, we import the `\"IO\"` namespace.\nThat namespace is a map of imports, in this case the `inspect` function, which is represented with a tuple of:\n\n1. the import type: `:fn` (a function),\n1. the functions parameter types: `[:i32]`,\n1. the functions return types: `[]`, and\n1. the function to be executed: `fn (_context, i) -> IO.inspect(i) end`\n\nThe first param the function receives is always the call context:\n\n    %{\n      memory: %Wasmex.Memory{},\n      caller: %Wasmex.StoreOrCaller{},\n      pid: pid(),\n      instance: %Wasmex.Instance{}\n    } = context\n\nThe `caller` MUST be used instead of a `store` in Wasmex API functions.\nWasmex might deadlock if the `store` is used instead of the `caller`\n(because running the Wasm instance holds a Mutex lock on the `store` so\nwe cannot use that store again during the execution of an imported function).\nThe caller, however, MUST NOT be used outside of the imported functions scope.\n\nAll other params are regular parameters as specified by the parameter type list.\n\nValid parameter/return types are:\n\n- `:i32` a 32 bit integer\n- `:i64` a 64 bit integer\n- `:v128` a 128 bit unsigned integer\n- `:f32` a 32 bit float\n- `:f64` a 64 bit float","title":"Imports - Wasmex.start_link/1","ref":"Wasmex.html#start_link/1-imports"},{"type":"function","doc":"Wasm module `links` may be given as an additional option.\nLinks is a map of module names to Wasm modules.\n\n    iex> calculator_wasm = File.read!(TestHelper.wasm_link_test_file_path())\n    iex> utils_wasm = File.read!(TestHelper.wasm_test_file_path())\n    iex> links = %{utils: %{bytes: utils_wasm}}\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: calculator_wasm, links: links})\n    iex> Wasmex.call_function(pid, \"sum_range\", [1, 5])\n    {:ok, [15]}\n\nIt is also possible to link an already compiled module.\nThis improves performance if the same module is used many times by compiling it only once.\n\n    iex> calculator_wasm = File.read!(TestHelper.wasm_link_test_file_path())\n    iex> utils_wasm = File.read!(TestHelper.wasm_test_file_path())\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, utils_module} = Wasmex.Module.compile(store, utils_wasm)\n    iex> links = %{utils: %{module: utils_module}}\n    iex> {:ok, pid} = Wasmex.start_link(%{bytes: calculator_wasm, links: links, store: store})\n    iex> Wasmex.call_function(pid, \"sum_range\", [1, 5])\n    {:ok, [15]}\n\n**Important:** Make sure to use the same store for the linked modules and the main module.\n\nWhen linking multiple Wasm modules, it is important to handle their dependencies properly.\nThis can be achieved by providing a map of module names to their respective Wasm modules in the `links` option.\n\nFor example, if we have a main module that depends on a calculator module, and the calculator module depends on a utils module, we can link them as follows:\n\n    iex> main_wasm = File.read!(TestHelper.wasm_link_dep_test_file_path())\n    iex> calculator_wasm = File.read!(TestHelper.wasm_link_test_file_path())\n    iex> utils_wasm = File.read!(TestHelper.wasm_test_file_path())\n    iex> links = %{\n    ...>   calculator: %{\n    ...>     bytes: calculator_wasm,\n    ...>     links: %{\n    ...>       utils: %{bytes: utils_wasm}\n    ...>     }\n    ...>   }\n    ...> }\n    iex> {:ok, _pid} = Wasmex.start_link(%{bytes: main_wasm, links: links})\n\nIn this example, the `links` map specifies that the `calculator` module depends on the `utils` module.\nThe `links` map is a nested map, where each module name is associated with a map that contains the Wasm module bytes and its dependencies.\n\nThe `links` map can also be used to link an already compiled module, as shown in the previous examples.","title":"Linking multiple Wasm modules - Wasmex.start_link/1","ref":"Wasmex.html#start_link/1-linking-multiple-wasm-modules"},{"type":"function","doc":"Optionally, modules can be run with WebAssembly System Interface (WASI) support.\nWASI functions are provided as native implementations by default but could be overridden\nwith Elixir provided functions.\n\n    iex> {:ok, _pid } = Wasmex.start_link(%{bytes: \"(module)\", wasi: true})\n\nIt is possible to overwrite the default WASI functions using the imports map:\n\n    iex> imports = %{\n    ...>   wasi_snapshot_preview1: %{\n    ...>     random_get: {:fn, [:i32, :i32], [:i32],\n    ...>                  fn %{memory: memory, caller: caller}, address, size ->\n    ...>                    Enum.each(0..size, fn index ->\n    ...>                      Wasmex.Memory.set_byte(caller, memory, address + index, 0)\n    ...>                    end)\n    ...>                    # We chose `4` as the random number with a fair dice roll\n    ...>                    Wasmex.Memory.set_byte(caller, memory, address, 4)\n    ...>                    0\n    ...>                  end\n    ...>                 }\n    ...>   }\n    ...> }\n    iex> {:ok, _pid} = Wasmex.start_link(%{bytes: \"(module)\", imports: imports})\n\nIn the example above, we overwrite the `random_get` function which is (as all other WASI functions)\nimplemented in Rust. This way our Elixir implementation of `random_get` is used instead of the\ndefault WASI implementation.\n\nOftentimes, WASI programs need additional inputs like environment variables, arguments,\nor file system access.\nThese can configured by additional `Wasmex.Wasi.WasiOptions`:\n\n    iex> wasi_options = %Wasmex.Wasi.WasiOptions{\n    ...>   args: [\"hello\", \"from elixir\"],\n    ...>   env: %{\n    ...>     \"A_NAME_MAPS\" => \"to a value\",\n    ...>     \"THE_TEST_WASI_FILE\" => \"prints all environment variables\"\n    ...>   },\n    ...>   preopen: [%Wasmex.Wasi.PreopenOptions{path: \"lib\", alias: \"src\"}]\n    ...> }\n    iex> {:ok, _pid} = Wasmex.start_link(%{bytes: \"(module)\", wasi: wasi_options})\n\nIt is also possible to capture stdout, stdin, or stderr of a WASI program using pipes:\n\n    iex> {:ok, stdin} = Wasmex.Pipe.new()\n    iex> {:ok, stdout} = Wasmex.Pipe.new()\n    iex> wasi_options = %Wasmex.Wasi.WasiOptions{\n    ...>   args: [\"wasmex\", \"echo\"],\n    ...>   stdin: stdin,\n    ...>   stdout: stdout\n    ...> }\n    iex> {:ok, pid } = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasi_test_file_path()), wasi: wasi_options})\n    iex> Wasmex.Pipe.write(stdin, \"Hey! It compiles! Ship it!\")\n    iex> Wasmex.Pipe.seek(stdin, 0)\n    iex> {:ok, _} = Wasmex.call_function(pid, :_start, [])\n    iex> Wasmex.Pipe.seek(stdout, 0)\n    iex> Wasmex.Pipe.read(stdout)\n    \"Hey! It compiles! Ship it!\\n\"\n\nIn the example above, we call a WASI program which echoes a line from stdin\nback to stdout.","title":"WASI - Wasmex.start_link/1","ref":"Wasmex.html#start_link/1-wasi"},{"type":"function","doc":"Returns the `Wasmex.Store` of the Wasm instance.","title":"Wasmex.store/1","ref":"Wasmex.html#store/1"},{"type":"function","doc":"iex> {:ok, pid} = Wasmex.start_link(%{bytes: File.read!(TestHelper.wasm_test_file_path())})\n    iex> {:ok, %Wasmex.StoreOrCaller{}} = Wasmex.store(pid)","title":"Example - Wasmex.store/1","ref":"Wasmex.html#store/1-example"},{"type":"module","doc":"This is the entry point to support for the [WebAssembly Component Model](https://component-model.bytecodealliance.org/).\n\nThe Component Model is a higher-level way to interact with WebAssembly modules that provides:\n- Better type safety through interface types\n- Standardized way to define imports and exports using WIT (WebAssembly Interface Types)\n- WASI support for system interface capabilities","title":"Wasmex.Components","ref":"Wasmex.Components.html"},{"type":"module","doc":"To use a WebAssembly component:\n\n1. Start a component instance:\n```elixir\n# Using raw bytes\nbytes = File.read!(\"path/to/component.wasm\")\n{:ok, pid} = Wasmex.Components.start_link(%{bytes: bytes})\n\n# Using a file path\n{:ok, pid} = Wasmex.Components.start_link(%{path: \"path/to/component.wasm\"})\n\n# With WASI support\n{:ok, pid} = Wasmex.Components.start_link(%{\n  path: \"path/to/component.wasm\",\n  wasi: %Wasmex.Wasi.WasiP2Options{}\n})\n\n# With imports (host functions the component can call)\n{:ok, pid} = Wasmex.Components.start_link(%{\n  bytes: bytes,\n  imports: %{\n    \"host_function\" => {:fn, &MyModule.host_function/1}\n  }\n})\n```\n\n2. Call exported functions:\n```elixir\n{:ok, result} = Wasmex.Components.call_function(pid, \"exported_function\", [\"param1\"])\n```","title":"Basic Usage - Wasmex.Components","ref":"Wasmex.Components.html#module-basic-usage"},{"type":"module","doc":"The component model supports the following WIT (WebAssembly Interface Type) types:","title":"Component Interface Types - Wasmex.Components","ref":"Wasmex.Components.html#module-component-interface-types"},{"type":"module","doc":"- **Primitive Types**\n  - Integers: `s8`, `s16`, `s32`, `s64`, `u8`, `u16`, `u32`, `u64`\n  - Floats: `f32`, `f64`\n  - `bool`\n  - `string`\n  - `char` (maps to Elixir strings with a single character)\n    ```wit\n    char\n    ```\n    ```elixir\n    \"A\"  # or from a code point\n    937  # Ω\n    ```\n\n- **Compound Types**\n  - `record` (maps to Elixir maps with atom keys)\n    ```wit\n    record point { x: u32, y: u32 }\n    ```\n    ```elixir\n    %{x: 1, y: 2}\n    ```\n\n  - `list ` (maps to Elixir lists)\n    ```wit\n    list \n    ```\n    ```elixir\n    [1, 2, 3]\n    ```\n\n  - `tuple ` (maps to Elixir tuples)\n    ```wit\n    tuple \n    ```\n    ```elixir\n    {1, \"two\"}\n    ```\n\n  - `option ` (maps to `nil` or the value)\n    ```wit\n    option \n    ```\n    ```elixir\n    nil  # or\n    42\n    ```\n\n  - `enum` (maps to Elixir atoms)\n    ```wit\n    enum size { s, m, l }\n    ```\n    ```elixir\n    :s  # or :m or :l\n    ```\n\n  - `variant` (tagged unions, maps to atoms or tuples)\n    ```wit\n    variant filter { all, none, lt(u32) }\n    ```\n    ```elixir\n    :all     # variant without payload\n    :none    # variant without payload\n    {:lt, 7} # variant with payload\n    ```\n\n  - `flags` (maps to Elixir maps with boolean values)\n    ```wit\n    flags permission { read, write, exec }\n    ```\n    ```elixir\n    %{read: true, write: true, exec: false}\n    # Note: When returned from WebAssembly, only the flags set to true are included\n    # %{read: true, exec: true}\n    ```\n\n  - `result ` (maps to Elixir tuples with :ok/:error)\n    ```wit\n    result \n    ```\n    ```elixir\n    {:ok, 42}      # success case\n    {:error, 404}  # error case\n    ```","title":"Supported Types - Wasmex.Components","ref":"Wasmex.Components.html#module-supported-types"},{"type":"module","doc":"The following WIT type is not yet supported:\n- Resources\n\nSupport for the Component Model should be considered beta quality.","title":"Currently Unsupported Types - Wasmex.Components","ref":"Wasmex.Components.html#module-currently-unsupported-types"},{"type":"module","doc":"The `start_link/1` function accepts the following options:\n\n* `:bytes` - Raw WebAssembly component bytes (mutually exclusive with `:path`)\n* `:path` - Path to a WebAssembly component file (mutually exclusive with `:bytes`)\n* `:wasi` - Optional WASI configuration as `Wasmex.Wasi.WasiP2Options` struct for system interface capabilities\n* `:imports` - Optional map of host functions that can be called by the WebAssembly component\n  * Keys are function names as strings\n  * Values are tuples of `{:fn, function}` where function is the host function to call\n\nAdditionally, any standard GenServer options (like `:name`) are supported.","title":"Options - Wasmex.Components","ref":"Wasmex.Components.html#module-options"},{"type":"module","doc":"```elixir\n# With raw bytes\n{:ok, pid} = Wasmex.Components.start_link(%{\n  bytes: File.read!(\"component.wasm\"),\n  name: MyComponent\n})\n\n# With WASI configuration\n{:ok, pid} = Wasmex.Components.start_link(%{\n  path: \"component.wasm\",\n  wasi: %Wasmex.Wasi.WasiP2Options{\n    allow_http: true\n  }\n})\n\n# With host functions\n{:ok, pid} = Wasmex.Components.start_link(%{\n  path: \"component.wasm\",\n  imports: %{\n    \"log\" => {:fn, &IO.puts/1},\n    \"add\" => {:fn, fn(a, b) -> a + b end}\n  }\n})\n```","title":"Examples - Wasmex.Components","ref":"Wasmex.Components.html#module-examples"},{"type":"function","doc":"","title":"Wasmex.Components.call_function/4","ref":"Wasmex.Components.html#call_function/4"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Wasmex.Components.child_spec/1","ref":"Wasmex.Components.html#child_spec/1"},{"type":"function","doc":"Starts a new WebAssembly component instance.","title":"Wasmex.Components.start_link/1","ref":"Wasmex.Components.html#start_link/1"},{"type":"function","doc":"* `:bytes` - Raw WebAssembly component bytes (mutually exclusive with `:path`)\n  * `:path` - Path to a WebAssembly component file (mutually exclusive with `:bytes`)\n  * `:wasi` - Optional WASI configuration as `Wasmex.Wasi.WasiP2Options` struct\n  * `:imports` - Optional map of host functions that can be called by the component\n  * Any standard GenServer options (like `:name`)","title":"Options - Wasmex.Components.start_link/1","ref":"Wasmex.Components.html#start_link/1-options"},{"type":"function","doc":"* `{:ok, pid}` on success\n  * `{:error, reason}` on failure","title":"Returns - Wasmex.Components.start_link/1","ref":"Wasmex.Components.html#start_link/1-returns"},{"type":"module","doc":"This represents a compiled but not yet instantiated WebAssembly component. It is\nanalogous to a Module in core webassembly.","title":"Wasmex.Components.Component","ref":"Wasmex.Components.Component.html"},{"type":"function","doc":"","title":"Wasmex.Components.Component.new/2","ref":"Wasmex.Components.Component.html#new/2"},{"type":"type","doc":"","title":"Wasmex.Components.Component.t/0","ref":"Wasmex.Components.Component.html#t:t/0"},{"type":"module","doc":"A GenServer wrapper for WebAssembly components. This module provides a macro to easily\ncreate GenServer-based components with wrapper functions for the exports in the WIT definition.","title":"Wasmex.Components.ComponentServer","ref":"Wasmex.Components.ComponentServer.html"},{"type":"module","doc":"To use this module, you need to:\n1. Create a WIT file defining your component's interface\n2. Create a module that uses ComponentServer with the path to your WIT file\n3. Use the generated functions to interact with your WebAssembly component","title":"Usage - Wasmex.Components.ComponentServer","ref":"Wasmex.Components.ComponentServer.html#module-usage"},{"type":"module","doc":"Given a WIT file `greeter.wit` with the following content:\n\n```wit\npackage example:greeter\n\nworld greeter {\n  export greet: func(who: string) -> string;\n  export multi-greet: func(who: string, times: u16) -> list ;\n}\n```\n\nYou can create a GenServer wrapper like this:\n\n```elixir\ndefmodule MyApp.Greeter do\n  use Wasmex.Components.ComponentServer,\n    wit: \"path/to/greeter.wit\"\nend\n```\n\nThis will automatically generate the following functions:\n\n```elixir\n# Start the component server\niex> {:ok, pid} = MyApp.Greeter.start_link(path: \"path/to/greeter.wasm\")\n\n# Generated function wrappers:\niex> MyApp.Greeter.greet(pid, \"World\")  # Returns: \"Hello, World!\"\niex> MyApp.Greeter.multi_greet(pid, \"World\", 2) # Returns: [\"Hello, World!\", \"Hello, World!\"]\n```","title":"Basic Example - Wasmex.Components.ComponentServer","ref":"Wasmex.Components.ComponentServer.html#module-basic-example"},{"type":"module","doc":"When your WebAssembly component imports functions, you can provide them using the `:imports` option.\nFor example, given a WIT file `logger.wit`:\n\n```wit\npackage example:logger\n\nworld logger {\n  import log: func(message: string)\n  import get-timestamp: func() -> u64\n\n  export log-with-timestamp: func(message: string)\n}\n```\n\nYou can implement the imported functions like this:\n\n```elixir\ndefmodule MyApp.Logger do\n  use Wasmex.Components.ComponentServer,\n    wit: \"path/to/logger.wit\",\n    imports: %{\n      \"log\" => fn message ->\n        IO.puts(message)\n        :ok\n      end,\n      \"get-timestamp\" => fn ->\n        System.system_time(:second)\n      end\n    }\nend\n```\n\n# Usage:\n```elixir\niex> {:ok, pid} = MyApp.Logger.start_link(wasm: \"path/to/logger.wasm\")\niex> MyApp.Logger.log_with_timestamp(pid, \"Hello from Wasm!\")\n```\n\nThe import functions should return the correct types as defined in the WIT file. Incorrect types will likely\ncause a crash, or possibly a NIF panic.","title":"Imports Example - Wasmex.Components.ComponentServer","ref":"Wasmex.Components.ComponentServer.html#module-imports-example"},{"type":"module","doc":"* `:wit` - Path to the WIT file defining the component's interface\n* `:imports` - A map of import function implementations that the component requires, where each key\n  is the function name as defined in the WIT file and the value is the implementing function","title":"Options - Wasmex.Components.ComponentServer","ref":"Wasmex.Components.ComponentServer.html#module-options"},{"type":"module","doc":"The component model equivalent to `Wasmex.Instance`","title":"Wasmex.Components.Instance","ref":"Wasmex.Components.Instance.html"},{"type":"function","doc":"","title":"Wasmex.Components.Instance.call_function/4","ref":"Wasmex.Components.Instance.html#call_function/4"},{"type":"function","doc":"","title":"Wasmex.Components.Instance.new/3","ref":"Wasmex.Components.Instance.html#new/3"},{"type":"module","doc":"This is the component model equivalent of `Wasmex.Store`","title":"Wasmex.Components.Store","ref":"Wasmex.Components.Store.html"},{"type":"function","doc":"","title":"Wasmex.Components.Store.new/2","ref":"Wasmex.Components.Store.html#new/2"},{"type":"function","doc":"","title":"Wasmex.Components.Store.new_wasi/3","ref":"Wasmex.Components.Store.html#new_wasi/3"},{"type":"module","doc":"An `Engine` which is a global context for compilation and management of Wasm\nmodules.\n\nEngines store global configuration preferences such as compilation settings,\nenabled features, etc. You'll likely only need at most one of these for a\nprogram.\n\nYou can create an engine with default configuration settings using\n`EngineConfig::default()`. Be sure to consult the documentation of\n`Wasmex.EngineConfig` for default settings.","title":"Wasmex.Engine","ref":"Wasmex.Engine.html"},{"type":"module","doc":"iex> {:ok, _engine} = Wasmex.Engine.new(%Wasmex.EngineConfig{})","title":"Example - Wasmex.Engine","ref":"Wasmex.Engine.html#module-example"},{"type":"function","doc":"Creates a new `Wasmex.Engine` with default settings.","title":"Wasmex.Engine.default/0","ref":"Wasmex.Engine.html#default/0"},{"type":"function","doc":"iex> _engine = Wasmex.Engine.default()","title":"Example - Wasmex.Engine.default/0","ref":"Wasmex.Engine.html#default/0-example"},{"type":"function","doc":"Creates a new `Wasmex.Engine` with the specified options.","title":"Wasmex.Engine.new/1","ref":"Wasmex.Engine.html#new/1"},{"type":"function","doc":"iex> {:ok, _engine} = Wasmex.Engine.new(%Wasmex.EngineConfig{})","title":"Example - Wasmex.Engine.new/1","ref":"Wasmex.Engine.html#new/1-example"},{"type":"function","doc":"Ahead-of-time (AOT) compiles a WebAssembly module.\n\nThe `bytes` provided must be in one of two formats:\n\n* A [binary-encoded][binary] WebAssembly module\n* A [text-encoded][text] instance of the WebAssembly text format\n\nThis method may be used to compile a module for use with a\ndifferent target host. The output of this method may be used with\n`Wasmex.Module.unsafe_deserialize/2` on hosts compatible with the\n`Wasmex.EngineConfig` associated with this `Wasmex.Engine`.\n\nThe output of this method is safe to send to another host machine\nfor later execution. As the output is already a compiled module,\ntranslation and code generation will be skipped and this will\nimprove the performance of constructing a `Wasmex.Module` from\nthe output of this method.\n\n[binary]: https://webassembly.github.io/spec/core/binary/index.html\n[text]: https://webassembly.github.io/spec/core/text/index.html","title":"Wasmex.Engine.precompile_module/2","ref":"Wasmex.Engine.html#precompile_module/2"},{"type":"function","doc":"iex> {:ok, engine} = Wasmex.Engine.new(%Wasmex.EngineConfig{})\n    iex> bytes = File.read!(TestHelper.wasm_test_file_path())\n    iex> {:ok, _serialized_module} = Wasmex.Engine.precompile_module(engine, bytes)","title":"Example - Wasmex.Engine.precompile_module/2","ref":"Wasmex.Engine.html#precompile_module/2-example"},{"type":"type","doc":"","title":"Wasmex.Engine.t/0","ref":"Wasmex.Engine.html#t:t/0"},{"type":"module","doc":"Configures a `Wasmex.Engine`.","title":"Wasmex.EngineConfig","ref":"Wasmex.EngineConfig.html"},{"type":"module","doc":"* `:consume_fuel` - Whether or not to consume fuel when executing Wasm instructions. This defaults to `false`.\n  * `:cranelift_opt_level` - Optimization level for the Cranelift code generator. This defaults to `:none`.\n  * `:wasm_backtrace_details` - Whether or not backtraces in traps will parse debug info in the Wasm file to have filename/line number information. This defaults to `false`.\n  * `:debug_info` - Configures whether DWARF debug information will be emitted during compilation. This defaults to `false`.\n  * `:memory64` - Whether or not to use 64-bit memory. This defaults to `false`.\n  * `:wasm_component_model` - Whether or not to use the WebAssembly component model. This defaults to `true`.","title":"Options - Wasmex.EngineConfig","ref":"Wasmex.EngineConfig.html#module-options"},{"type":"module","doc":"iex> _config = %Wasmex.EngineConfig{}\n    ...>           |> Wasmex.EngineConfig.consume_fuel(true)\n    ...>           |> Wasmex.EngineConfig.cranelift_opt_level(:speed)\n    ...>           |> Wasmex.EngineConfig.wasm_backtrace_details(false)","title":"Example - Wasmex.EngineConfig","ref":"Wasmex.EngineConfig.html#module-example"},{"type":"function","doc":"Configures whether execution of WebAssembly will \"consume fuel\" to\neither halt or yield execution as desired.\n\nThis can be used to deterministically prevent infinitely-executing\nWebAssembly code by instrumenting generated code to consume fuel as it\nexecutes. When fuel runs out a trap is raised.\n\nNote that a `Wasmex.Store` starts with no fuel, so if you enable this option\nyou'll have to be sure to pour some fuel into `Wasmex.Store` before\nexecuting some code. See `Wasmex.StoreOrCaller.set_fuel/2`.","title":"Wasmex.EngineConfig.consume_fuel/2","ref":"Wasmex.EngineConfig.html#consume_fuel/2"},{"type":"function","doc":"iex> config = %Wasmex.EngineConfig{}\n    ...>          |> Wasmex.EngineConfig.consume_fuel(true)\n    iex> config.consume_fuel\n    true","title":"Example - Wasmex.EngineConfig.consume_fuel/2","ref":"Wasmex.EngineConfig.html#consume_fuel/2-example"},{"type":"function","doc":"Configures the Cranelift code generator optimization level.\n\nAllows one of the following values:\n\n* `:none` - No optimizations performed, minimizes compilation time by disabling most optimizations.\n* `:speed` - Generates the fastest possible code, but may take longer.\n* `:speed_and_size` - Similar to `speed`, but also performs transformations aimed at reducing code size.","title":"Wasmex.EngineConfig.cranelift_opt_level/2","ref":"Wasmex.EngineConfig.html#cranelift_opt_level/2"},{"type":"function","doc":"Configures whether the WebAssembly memory type is 64-bit.","title":"Wasmex.EngineConfig.memory64/2","ref":"Wasmex.EngineConfig.html#memory64/2"},{"type":"function","doc":"iex> config = %Wasmex.EngineConfig{}\n    ...>          |> Wasmex.EngineConfig.memory64(true)\n    iex> config.memory64\n    true","title":"Example - Wasmex.EngineConfig.memory64/2","ref":"Wasmex.EngineConfig.html#memory64/2-example"},{"type":"type","doc":"","title":"Wasmex.EngineConfig.t/0","ref":"Wasmex.EngineConfig.html#t:t/0"},{"type":"function","doc":"Configures whether backtraces in traps will parse debug info in the Wasm\nfile to have filename/line number information.\n\nWhen enabled this will causes modules to retain debugging information\nfound in Wasm binaries. This debug information will be used when a trap\nhappens to symbolicate each stack frame and attempt to print a\nfilename/line number for each Wasm frame in the stack trace.","title":"Wasmex.EngineConfig.wasm_backtrace_details/2","ref":"Wasmex.EngineConfig.html#wasm_backtrace_details/2"},{"type":"function","doc":"iex> config = %Wasmex.EngineConfig{}\n    ...>          |> Wasmex.EngineConfig.wasm_backtrace_details(true)\n    iex> config.wasm_backtrace_details\n    true","title":"Example - Wasmex.EngineConfig.wasm_backtrace_details/2","ref":"Wasmex.EngineConfig.html#wasm_backtrace_details/2-example"},{"type":"module","doc":"Instantiates a Wasm module and allows calling exported functions on it.\n\nIn the majority of cases, you will not need to use this module directly\nbut use the main module `Wasmex` instead.\nThis module expects to be executed within GenServer context which `Wasmex` sets up.","title":"Wasmex.Instance","ref":"Wasmex.Instance.html"},{"type":"function","doc":"Calls a function the given `name` exported by the Wasm `instance` with the given `params`.\n\nThe Wasm function will be invoked asynchronously in a new OS thread.\nThe calling Process/GenServer will receive a `{:returned_function_call, result, from}`\nmessage once execution finishes.\nThe result either is an `{:error, reason}` or `:ok`.\n\n`call_exported_function/5` assumes to be called within a GenServer context, it expects a `from` argument\nas given by `c:GenServer.handle_call/3`. `from` is returned unchanged to allow\nthe wrapping GenServer to reply to their caller.\n\nA BadArg exception may be thrown when given unexpected input data.","title":"Wasmex.Instance.call_exported_function/5","ref":"Wasmex.Instance.html#call_exported_function/5"},{"type":"function","doc":"Parameters for Wasm functions are automatically casted to Wasm values.\nNote that WebAssembly only knows number datatypes (floats and integers of various sizes).\n\nYou can pass arbitrary data to WebAssembly by writing that data into an instances `Wasmex.Memory`.\nThe `memory/2` function returns the instances memory.","title":"Function parameters - Wasmex.Instance.call_exported_function/5","ref":"Wasmex.Instance.html#call_exported_function/5-function-parameters"},{"type":"function","doc":"iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> Wasmex.Instance.call_exported_function(store, instance, \"sum\", [1, 2], :from)\n    :ok\n    iex> receive do\n    ...>   {:returned_function_call, {:ok, [3]}, :from} -> :ok\n    ...> after\n    ...>  1000 -> raise \"message_expected\"\n    ...> end","title":"Example - Wasmex.Instance.call_exported_function/5","ref":"Wasmex.Instance.html#call_exported_function/5-example"},{"type":"function","doc":"Whether the Wasm `instance` exports a function with the given `name`.","title":"Wasmex.Instance.function_export_exists/3","ref":"Wasmex.Instance.html#function_export_exists/3"},{"type":"function","doc":"iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> Wasmex.Instance.function_export_exists(store, instance, \"sum\")\n    true\n    iex> Wasmex.Instance.function_export_exists(store, instance, \"does_not_exist\")\n    false","title":"Example - Wasmex.Instance.function_export_exists/3","ref":"Wasmex.Instance.html#function_export_exists/3-example"},{"type":"function","doc":"Reads the value of an exported global.","title":"Wasmex.Instance.get_global_value/3","ref":"Wasmex.Instance.html#get_global_value/3"},{"type":"function","doc":"iex> wat = \"(module\n    ...>          (global $answer i32 (i32.const 42))\n    ...>          (export \\\"answer\\\" (global $answer))\n    ...>        )\"\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, module} = Wasmex.Module.compile(store, wat)\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> Wasmex.Instance.get_global_value(store, instance, \"answer\")\n    {:ok, 42}\n    iex> Wasmex.Instance.get_global_value(store, instance, \"not_a_global\")\n    {:error, \"exported global `not_a_global` not found\"}","title":"Examples - Wasmex.Instance.get_global_value/3","ref":"Wasmex.Instance.html#get_global_value/3-examples"},{"type":"function","doc":"Returns the `Wasmex.Memory` of the Wasm `instance`.","title":"Wasmex.Instance.memory/2","ref":"Wasmex.Instance.html#memory/2"},{"type":"function","doc":"iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, %Wasmex.Memory{}} = Wasmex.Instance.memory(store, instance)","title":"Example - Wasmex.Instance.memory/2","ref":"Wasmex.Instance.html#memory/2-example"},{"type":"function","doc":"Instantiates a Wasm module with the given imports.\n\nReturns the instantiated Wasm instance.\n\nThe `import` parameter is a nested map of Wasm namespaces.\nEach namespace consists of a name and a map of function names to function signatures.\n\nThe `links` parameter is a list of name-module pairs that are dynamically linked to the instance.\n\nFunction signatures are a tuple of the form `{:fn, arg_types, return_types, callback}`.\nWhere `arg_types` and `return_types` are lists of `:i32`, `:i64`, `:f32`, `:f64`, `:v128`.\n\nEach `callback` function receives a `context` map as the first argument followed by the arguments specified in its signature.\n`context` has the following keys:\n\n  * `:memory` - The default exported `Wasmex.Memory` of the Wasm instance\n  * `:caller` - The caller of the Wasm instance which MUST be used instead of a `Wasmex.Store` in all Wasmex functions called from within the callback. Failure to do so will result in a deadlock. The `caller` MUST NOT be used outside of the callback.","title":"Wasmex.Instance.new/4","ref":"Wasmex.Instance.html#new/4"},{"type":"function","doc":"This example instantiates a Wasm module with one namespace `env` having\nthree imported functions `imported_sum3`, `imported_sumf`, and `imported_void`.\n\nThe imported function `imported_sum3` takes three `:i32` (32 bit integer) arguments and returns a `:i32` number.\nIts implementation is defined by the callback function `fn _context, a, b, c -> a + b + c end`.\n\n    iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> imports = %{\n    ...>   \"env\" =>\n    ...>     %{\n    ...>       \"imported_sum3\" => {:fn, [:i32, :i32, :i32], [:i32], fn _context, a, b, c -> a + b + c end},\n    ...>       \"imported_sumf\" => {:fn, [:f32, :f32], [:f32], fn _context, a, b -> a + b end},\n    ...>       \"imported_void\" => {:fn, [], [], fn _context -> nil end}\n    ...>     }\n    ...> }\n    ...> links = []\n    iex> {:ok, %Wasmex.Instance{}} = Wasmex.Instance.new(store, module, imports, links)","title":"Example - Wasmex.Instance.new/4","ref":"Wasmex.Instance.html#new/4-example"},{"type":"function","doc":"Sets the value of an exported mutable global.","title":"Wasmex.Instance.set_global_value/4","ref":"Wasmex.Instance.html#set_global_value/4"},{"type":"function","doc":"iex> wat = \"(module\n    ...>          (global $count (mut i32) (i32.const 0))\n    ...>          (export \\\"count\\\" (global $count))\n    ...>        )\"\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, module} = Wasmex.Module.compile(store, wat)\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> Wasmex.Instance.set_global_value(store, instance, \"count\", 1)\n    :ok\n    iex> Wasmex.Instance.get_global_value(store, instance, \"count\")\n    {:ok, 1}","title":"Examples - Wasmex.Instance.set_global_value/4","ref":"Wasmex.Instance.html#set_global_value/4-examples"},{"type":"type","doc":"","title":"Wasmex.Instance.t/0","ref":"Wasmex.Instance.html#t:t/0"},{"type":"module","doc":"Memory is a linear array of bytes to store Wasm values. The `Memory` module provides functions to read and write to this array.\n\n`Memory` is accessible through `Wasmex.Instance.memory/2`,\n`Wasmex.Memory.from_instance/2`, or as the caller context\nof an imported function (see `Wasmex.Instance.call_exported_function/5`).\n\n    iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Instance.memory(store, instance)\n    iex> Wasmex.Memory.set_byte(store, memory, 0, 42)\n    iex> Wasmex.Memory.get_byte(store, memory, 0)\n    42\n\nWasm memory is organized in pages of 64kb and may be grown by additional pages.","title":"Wasmex.Memory","ref":"Wasmex.Memory.html"},{"type":"function","doc":"Returns the exported memory resource of the given `Wasmex.Instance`.","title":"Wasmex.Memory.from_instance/2","ref":"Wasmex.Memory.html#from_instance/2"},{"type":"function","doc":"iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, %Wasmex.Memory{}} = Wasmex.Memory.from_instance(store, instance)","title":"Example - Wasmex.Memory.from_instance/2","ref":"Wasmex.Memory.html#from_instance/2-example"},{"type":"function","doc":"Returns the byte at the given `index`.","title":"Wasmex.Memory.get_byte/3","ref":"Wasmex.Memory.html#get_byte/3"},{"type":"function","doc":"Set a value at memory position `0` and read it back:\n\n    iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Memory.from_instance(store, instance)\n    iex> Wasmex.Memory.set_byte(store, memory, 0, 42)\n    iex> Wasmex.Memory.get_byte(store, memory, 0)\n    42","title":"Example - Wasmex.Memory.get_byte/3","ref":"Wasmex.Memory.html#get_byte/3-example"},{"type":"function","doc":"Grows the amount of available memory by the given number of pages.\n\nReturns the number of previously available pages.\nA page has a size of 64 kB or 65,536 bytes.\n\nReturns an error if memory could not be grown.","title":"Wasmex.Memory.grow/3","ref":"Wasmex.Memory.html#grow/3"},{"type":"function","doc":"iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Memory.from_instance(store, instance)\n    iex> Wasmex.Memory.grow(store, memory, 1)\n    17","title":"Example - Wasmex.Memory.grow/3","ref":"Wasmex.Memory.html#grow/3-example"},{"type":"function","doc":"Reads the given number of bytes from the given memory at the given index.\n\nReturns the read bytes as a binary.","title":"Wasmex.Memory.read_binary/4","ref":"Wasmex.Memory.html#read_binary/4"},{"type":"function","doc":"Reads 5 bytes from memory position `0`, given it contains the 5 ASCII\ncharacters forming \"hello\".\n\n    iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Memory.from_instance(store, instance)\n    iex> Wasmex.Memory.write_binary(store, memory, 0, \"hello\")\n    iex> Wasmex.Memory.read_binary(store, memory, 0, 5)\n    \"hello\"\n    iex> Wasmex.Memory.read_binary(store, memory, 3, 2)\n    \"lo\"","title":"Example - Wasmex.Memory.read_binary/4","ref":"Wasmex.Memory.html#read_binary/4-example"},{"type":"function","doc":"Reads the given number of bytes from the given memory at the given index.\n\nReturns the read bytes as a string.","title":"Wasmex.Memory.read_string/4","ref":"Wasmex.Memory.html#read_string/4"},{"type":"function","doc":"Reads 5 bytes from memory position `0`, given it contains the 5 ASCII\ncharacters forming \"hello\".\n\n    iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Memory.from_instance(store, instance)\n    iex> Wasmex.Memory.write_binary(store, memory, 0, \"hello\")\n    iex> Wasmex.Memory.read_string(store, memory, 0, 5)\n    \"hello\"\n    iex> Wasmex.Memory.read_string(store, memory, 3, 2)\n    \"lo\"","title":"Example - Wasmex.Memory.read_string/4","ref":"Wasmex.Memory.html#read_string/4-example"},{"type":"function","doc":"Sets the byte at the given `index` to the given `value`.","title":"Wasmex.Memory.set_byte/4","ref":"Wasmex.Memory.html#set_byte/4"},{"type":"function","doc":"Set a value at memory position `0` and read it back:\n\n    iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Memory.from_instance(store, instance)\n    iex> Wasmex.Memory.set_byte(store, memory, 0, 42)\n    iex> Wasmex.Memory.get_byte(store, memory, 0)\n    42","title":"Example - Wasmex.Memory.set_byte/4","ref":"Wasmex.Memory.html#set_byte/4-example"},{"type":"function","doc":"Returns the size in bytes of the given memory.\n\nNote that the size of the memory is always a multiple of 64kb (one page).","title":"Wasmex.Memory.size/2","ref":"Wasmex.Memory.html#size/2"},{"type":"function","doc":"iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Memory.from_instance(store, instance)\n    iex> Wasmex.Memory.size(store, memory)\n    1114112 # in bytes (17 pages of 64 kB)","title":"Example - Wasmex.Memory.size/2","ref":"Wasmex.Memory.html#size/2-example"},{"type":"type","doc":"","title":"Wasmex.Memory.t/0","ref":"Wasmex.Memory.html#t:t/0"},{"type":"function","doc":"Writes the given `binary` into the memory at the given `index`.","title":"Wasmex.Memory.write_binary/4","ref":"Wasmex.Memory.html#write_binary/4"},{"type":"function","doc":"Writes 5 bytes representing the ASCII characters for \"hello\"\nat memory position `0`.\n\n    iex> %{store: store, module: module} = TestHelper.wasm_module()\n    iex> {:ok, instance} = Wasmex.Instance.new(store, module, %{})\n    iex> {:ok, memory} = Wasmex.Memory.from_instance(store, instance)\n    iex> Wasmex.Memory.write_binary(store, memory, 0, \"hello\")\n    :ok","title":"Example - Wasmex.Memory.write_binary/4","ref":"Wasmex.Memory.html#write_binary/4-example"},{"type":"module","doc":"A compiled WebAssembly module.\n\nA Wasm Module contains stateless WebAssembly code that has\nalready been compiled and can be instantiated multiple times.","title":"Wasmex.Module","ref":"Wasmex.Module.html"},{"type":"function","doc":"Compiles a Wasm module from it's Wasm (a .wasm file) or WAT (a .wat file) representation.\n\nCompiled modules can be instantiated using `Wasmex.start_link/1` or `Instance.new/3`.\n\nSince module compilation takes time and resources but instantiation is\ncomparatively cheap, it may be a good idea to compile a module once and\ninstantiate it often if you want to run a Wasm binary multiple times.","title":"Wasmex.Module.compile/2","ref":"Wasmex.Module.html#compile/2"},{"type":"function","doc":"Read a Wasm file and compile it into a Wasm module.\nUse the compiled module to start a running `Wasmex.Instance`.\n\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, module} = Wasmex.Module.compile(store, File.read!(TestHelper.wasm_test_file_path()))\n    iex> {:ok, _pid} = Wasmex.start_link(%{store: store, module: module})\n\nModules can be compiled from WAT (WebAssembly Text) format:\n\n    iex> wat = \"(module)\" # minimal and not very useful\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, %Wasmex.Module{}} = Wasmex.Module.compile(store, wat)","title":"Example - Wasmex.Module.compile/2","ref":"Wasmex.Module.html#compile/2-example"},{"type":"function","doc":"Lists all exports of a Wasm module.\n\nReturns a map which has the exports name (string) as key and export info-tuples as values.\nInfo tuples always start with an atom indicating the exports type:\n\n* `:fn` (function)\n* `:global`\n* `:table`\n* `:memory`\n\nFurther parts of the info tuple vary depending on the type.","title":"Wasmex.Module.exports/1","ref":"Wasmex.Module.html#exports/1"},{"type":"function","doc":"List the exported function \"hello_world()\" of a Wasm module:\n\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> wat = \"(module\n    ...>          (func $helloWorld (result i32) (i32.const 42))\n    ...>          (export \\\"hello_world\\\" (func $helloWorld))\n    ...>        )\"\n    iex> {:ok, module} = Wasmex.Module.compile(store, wat)\n    iex> Wasmex.Module.exports(module)\n    %{\n      \"hello_world\" => {:fn, [], [:i32]},\n    }","title":"Example - Wasmex.Module.exports/1","ref":"Wasmex.Module.html#exports/1-example"},{"type":"function","doc":"Lists all imports of a WebAssembly module grouped by their module namespace.\n\nReturns a map of namespace names to namespaces with each namespace being a map again.\nA namespace is a map of imports with the import name as key and and info-tuple as value.\n\nInfo tuples always start with an atom indicating the imports type:\n\n* `:fn` (function)\n* `:global`\n* `:table`\n* `:memory`\n\nFurther parts of the info tuple vary depending on the type.","title":"Wasmex.Module.imports/1","ref":"Wasmex.Module.html#imports/1"},{"type":"function","doc":"Show that the Wasm module imports a function \"inspect\" from the \"IO\" namespace:\n\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> wat = \"(module\n    ...>          (import \\\"IO\\\" \\\"inspect\\\" (func $log (param i32)))\n    ...>        )\"\n    iex> {:ok, module} = Wasmex.Module.compile(store, wat)\n    iex> Wasmex.Module.imports(module)\n    %{\n      \"IO\" => %{\n        \"inspect\" => {:fn, [:i32], []},\n      }\n    }","title":"Example - Wasmex.Module.imports/1","ref":"Wasmex.Module.html#imports/1-example"},{"type":"function","doc":"Returns the name of the current module if a name is given.\n\nThis name is normally set in the Wasm bytecode by some compilers.","title":"Wasmex.Module.name/1","ref":"Wasmex.Module.html#name/1"},{"type":"function","doc":"iex> {:ok, store} = Wasmex.Store.new()\n    iex> wat = \"(module $hiFromTheDocs)\" # minimal and not very useful Wasm module\n    iex> {:ok, module} = Wasmex.Module.compile(store, wat)\n    iex> Wasmex.Module.name(module)\n    \"hiFromTheDocs\"","title":"Example - Wasmex.Module.name/1","ref":"Wasmex.Module.html#name/1-example"},{"type":"function","doc":"Serializes a compiled Wasm module into a binary.\n\nThe generated binary can be deserialized back into a module using `unsafe_deserialize/1`.\nIt is unsafe do alter the binary in any way. See `unsafe_deserialize/1` for safety considerations.","title":"Wasmex.Module.serialize/1","ref":"Wasmex.Module.html#serialize/1"},{"type":"function","doc":"Serializes a compiled module:\n\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, module} = Wasmex.Module.compile(store, File.read!(TestHelper.wasm_test_file_path()))\n    iex> {:ok, serialized} = Wasmex.Module.serialize(module)\n    iex> is_binary(serialized)\n    true","title":"Example - Wasmex.Module.serialize/1","ref":"Wasmex.Module.html#serialize/1-example"},{"type":"type","doc":"","title":"Wasmex.Module.t/0","ref":"Wasmex.Module.html#t:t/0"},{"type":"function","doc":"Deserializes an in-memory compiled module previously created with\n`Wasmex.Module.serialize/1` or `Wasmex.Engine::precompile_module/2`.\n\nThis function will deserialize the binary blobs emitted by\n`Wasmex.Module.serialize/1` and `Wasmex.Engine::precompile_module/2`\nback into an in-memory `Wasmex.Module` that's ready to be instantiated.\n\n# Unsafety\n\nThis function is marked as `unsafe` because if fed invalid input or used\nimproperly this could lead to memory safety vulnerabilities. This method\nshould not, for example, be exposed to arbitrary user input.\n\nThe structure of the binary blob read here is only lightly validated\ninternally in `wasmtime`. This is intended to be an efficient\n\"rehydration\" for a `Wasmex.Module` which has very few runtime checks\nbeyond deserialization. Arbitrary input could, for example, replace\nvalid compiled code with any other valid compiled code, meaning that\nthis can trivially be used to execute arbitrary code otherwise.\n\nFor these reasons this function is `unsafe`. This function is only\ndesigned to receive the previous input from `Wasmex.Module.serialize/1`\nand `Wasmex.Engine::precompile_module/2`. If the exact output of those\nfunctions (unmodified) is passed to this function then calls to this\nfunction can be considered safe. It is the caller's responsibility to\nprovide the guarantee that only previously-serialized bytes are being\npassed in here.\n\nNote that this function is designed to be safe receiving output from\n*any* compiled version of `wasmtime` itself. This means that it is safe\nto feed output from older versions of Wasmtime into this function, in\naddition to newer versions of wasmtime (from the future!). These inputs\nwill deterministically and safely produce an error. This function only\nsuccessfully accepts inputs from the same version of `wasmtime`.\n(this means that if you cache blobs across versions of wasmtime you can\nbe safely guaranteed that future versions of wasmtime will reject old\ncache entries).\n\nIt is best to deserialize modules using an Engine with the same\nconfiguration as the one used to serialize/precompile it.","title":"Wasmex.Module.unsafe_deserialize/2","ref":"Wasmex.Module.html#unsafe_deserialize/2"},{"type":"function","doc":"Serializes a compiled module and deserializes it again:\n\n    iex> {:ok, store} = Wasmex.Store.new()\n    iex> {:ok, module} = Wasmex.Module.compile(store, File.read!(TestHelper.wasm_test_file_path()))\n    iex> {:ok, serialized} = Wasmex.Module.serialize(module)\n    iex> {:ok, %Wasmex.Module{}} = Wasmex.Module.unsafe_deserialize(serialized)","title":"Example - Wasmex.Module.unsafe_deserialize/2","ref":"Wasmex.Module.html#unsafe_deserialize/2-example"},{"type":"module","doc":"A Pipe is a memory buffer that can be used in exchange for a Wasm file.\n\nPipes have a read and write position which can be set using `seek/2`.","title":"Wasmex.Pipe","ref":"Wasmex.Pipe.html"},{"type":"module","doc":"Pipes can be written to and read from:\n\n    iex> {:ok, pipe} = Wasmex.Pipe.new()\n    iex> Wasmex.Pipe.write(pipe, \"hello\")\n    {:ok, 5}\n    iex> Wasmex.Pipe.seek(pipe, 0)\n    iex> Wasmex.Pipe.read(pipe)\n    \"hello\"\n\nThey can be used to capture stdout/stdin/stderr of WASI programs:\n\n    iex> {:ok, stdin} = Wasmex.Pipe.new()\n    iex> {:ok, stdout} = Wasmex.Pipe.new()\n    iex> {:ok, stderr} = Wasmex.Pipe.new()\n    iex> Wasmex.Store.new_wasi(%Wasmex.Wasi.WasiOptions{\n    ...>   stdin: stdin,\n    ...>   stdout: stdout,\n    ...>   stderr: stderr,\n    ...> })","title":"Example - Wasmex.Pipe","ref":"Wasmex.Pipe.html#module-example"},{"type":"function","doc":"Creates and returns a new Pipe.","title":"Wasmex.Pipe.new/0","ref":"Wasmex.Pipe.html#new/0"},{"type":"function","doc":"iex> {:ok, %Pipe{}} = Wasmex.Pipe.new()","title":"Example - Wasmex.Pipe.new/0","ref":"Wasmex.Pipe.html#new/0-example"},{"type":"function","doc":"Reads all available bytes from the Pipe and returns them as a binary.\n\nThis function does not block if there are no bytes available.\nReading starts at the current read position, see `seek/2`, and forwards the read position to the end of the Pipe.\nThe read bytes are not erased and can be read again after seeking back.","title":"Wasmex.Pipe.read/1","ref":"Wasmex.Pipe.html#read/1"},{"type":"function","doc":"iex> {:ok, pipe} = Wasmex.Pipe.new()\n    iex> Wasmex.Pipe.write(pipe, \"hello\")\n    iex> Wasmex.Pipe.read(pipe) # current position is at EOL, nothing more to read\n    \"\"\n    iex> Wasmex.Pipe.seek(pipe, 0)\n    iex> Wasmex.Pipe.read(pipe)\n    \"hello\"\n    iex> Wasmex.Pipe.seek(pipe, 3)\n    iex> Wasmex.Pipe.read(pipe)\n    \"lo\"\n    iex> Wasmex.Pipe.read(pipe)\n    \"\"","title":"Example - Wasmex.Pipe.read/1","ref":"Wasmex.Pipe.html#read/1-example"},{"type":"function","doc":"Sets the read/write position of the Pipe to the given position.\n\nThe position is given as a number of bytes from the start of the Pipe.","title":"Wasmex.Pipe.seek/2","ref":"Wasmex.Pipe.html#seek/2"},{"type":"function","doc":"iex> {:ok, pipe} = Wasmex.Pipe.new()\n    iex> Wasmex.Pipe.write(pipe, \"hello\")\n    iex> Wasmex.Pipe.seek(pipe, 0)\n    :ok\n    iex> Wasmex.Pipe.read(pipe)\n    \"hello\"","title":"Example - Wasmex.Pipe.seek/2","ref":"Wasmex.Pipe.html#seek/2-example"},{"type":"function","doc":"Returns the current size of the Pipe in bytes.","title":"Wasmex.Pipe.size/1","ref":"Wasmex.Pipe.html#size/1"},{"type":"function","doc":"iex> {:ok, pipe} = Wasmex.Pipe.new()\n    iex> Wasmex.Pipe.size(pipe)\n    0\n    iex> Wasmex.Pipe.write(pipe, \"hello\")\n    iex> Wasmex.Pipe.size(pipe)\n    5","title":"Example - Wasmex.Pipe.size/1","ref":"Wasmex.Pipe.html#size/1-example"},{"type":"type","doc":"","title":"Wasmex.Pipe.t/0","ref":"Wasmex.Pipe.html#t:t/0"},{"type":"function","doc":"Writes the given binary into the pipe.\n\nWriting starts at the current write position, see `seek/2`, and forwards it.","title":"Wasmex.Pipe.write/2","ref":"Wasmex.Pipe.html#write/2"},{"type":"function","doc":"iex> {:ok, pipe} = Wasmex.Pipe.new()\n    iex> Wasmex.Pipe.write(pipe, \"hello\")\n    {:ok, 5}\n    iex> Wasmex.Pipe.seek(pipe, 0)\n    iex> Wasmex.Pipe.read(pipe)\n    \"hello\"","title":"Example - Wasmex.Pipe.write/2","ref":"Wasmex.Pipe.html#write/2-example"},{"type":"module","doc":"A Store is a collection of Wasm instances and host-defined state.\n\nAll Wasm instances and items will be attached to and refer to a Store.\nFor example instances, functions, globals, and tables are all attached to a Store.\nInstances are created by instantiating a Module within a Store.\nMany functions of the Wasmex API require a Store in the form of a `Wasmex.StoreOrCaller`\nto be passed in.\n\nA Store is intended to be a short-lived object in a program. No form of GC is\nimplemented at this time so once an instance is created within a Store it will\nnot be deallocated until the Store itself is garbage collected. This makes Store\nunsuitable for creating an unbounded number of instances in it because Store will\nnever release this memory. It’s recommended to have a Store correspond roughly\nto the lifetime of a “main instance”.","title":"Wasmex.Store","ref":"Wasmex.Store.html"},{"type":"function","doc":"Creates a new Wasm store.\n\nReturns a `Wasmex.StoreOrCaller` even though we know it’s definitely a Store.\nThis allows Elixir-provided imported functions, which only have a \"Caller\", to use the same Wasmex APIs.\n\nOptionally, a `Wasmex.StoreLimits` can be passed in to limit the resources that can be created within the store.","title":"Wasmex.Store.new/2","ref":"Wasmex.Store.html#new/2"},{"type":"function","doc":"iex> {:ok, %Wasmex.StoreOrCaller{}} = Wasmex.Store.new()\n\n    iex> limits = %Wasmex.StoreLimits{memory_size: 1_000_000}\n    iex> {:ok, %Wasmex.StoreOrCaller{}} = Wasmex.Store.new(limits)","title":"Examples - Wasmex.Store.new/2","ref":"Wasmex.Store.html#new/2-examples"},{"type":"function","doc":"Creates a new Wasm store with WASI support.\n\nReturns a `Wasmex.StoreOrCaller` even though we know it’s definitely a Store.\nThis allows Elixir-provided imported functions, which only have a \"Caller\", to use the same Wasmex APIs.\n\nSee `Wasmex.Wasi.WasiOptions` for WASI configuration options.","title":"Wasmex.Store.new_wasi/3","ref":"Wasmex.Store.html#new_wasi/3"},{"type":"function","doc":"iex> {:ok, %Wasmex.StoreOrCaller{}} = Wasmex.Store.new_wasi(%Wasmex.Wasi.WasiOptions{})\n\n    iex> wasi_opts = %Wasmex.Wasi.WasiOptions{args: [\"arg1\", \"arg2\"]}\n    iex> limits = %Wasmex.StoreLimits{memory_size: 1_000_000}\n    iex> {:ok, %Wasmex.StoreOrCaller{}} = Wasmex.Store.new_wasi(wasi_opts, limits)","title":"Examples - Wasmex.Store.new_wasi/3","ref":"Wasmex.Store.html#new_wasi/3-examples"},{"type":"module","doc":"Configures limits to limit resource creation within a `Wasmex.Store`.\n\nWhenever resources such as linear memory, tables, or instances are\nallocated the `StoreLimits` specified here are enforced.","title":"Wasmex.StoreLimits","ref":"Wasmex.StoreLimits.html"},{"type":"module","doc":"* `:memory_size` - The maximum number of bytes a linear memory can grow to. Growing a linear memory beyond this limit will fail. By default, linear memory will not be limited.\n  * `:table_elements` - The maximum number of elements in a table. Growing a table beyond this limit will fail. By default, table elements will not be limited.\n  * `:instances` - The maximum number of instances that can be created for a `Wasmex.Store`. Module instantiation will fail if this limit is exceeded. This value defaults to 10,000.\n  * `:tables` - The maximum number of tables that can be created for a `Wasmex.Store`. Module instantiation will fail if this limit is exceeded. This value defaults to 10,000.\n  * `:memories` - The maximum number of linear memories that can be created for a `Wasmex.Store`. Instantiation will fail with an error if this limit is exceeded. This value defaults to 10,000.","title":"Options - Wasmex.StoreLimits","ref":"Wasmex.StoreLimits.html#module-options"},{"type":"module","doc":"iex> Wasmex.Store.new(%Wasmex.StoreLimits{\n    ...>   memory_size: 1_000_000,\n    ...>   table_elements: 100_000,\n    ...>   instances: 2,\n    ...>   tables: 10\n    ...>   memories: 10\n    ...> })","title":"Example - Wasmex.StoreLimits","ref":"Wasmex.StoreLimits.html#module-example"},{"type":"type","doc":"","title":"Wasmex.StoreLimits.t/0","ref":"Wasmex.StoreLimits.html#t:t/0"},{"type":"module","doc":"Either a `Wasmex.Store` or \"Caller\" for imported functions.\n\nA Store is a collection of Wasm instances and host-defined state, see `Wasmex.Store`.\nA Caller takes the place of a Store in imported function calls. If a Store is needed in\nElixir-provided imported functions, always use the provided Caller because\nusing the Store will cause a deadlock (the running Wasm instance locks the Stores Mutex).\n\nWhen configured, a StoreOrCaller can consume fuel to halt or yield execution as desired.\nSee `Wasmex.EngineConfig.consume_fuel/2` for more information on fuel consumption.","title":"Wasmex.StoreOrCaller","ref":"Wasmex.StoreOrCaller.html"},{"type":"function","doc":"Returns the amount of fuel available for future execution of this store.","title":"Wasmex.StoreOrCaller.get_fuel/1","ref":"Wasmex.StoreOrCaller.html#get_fuel/1"},{"type":"function","doc":"iex> {:ok, engine} = Wasmex.Engine.new(%Wasmex.EngineConfig{consume_fuel: true})\n    iex> {:ok, store} = Wasmex.Store.new(nil, engine)\n    iex> Wasmex.StoreOrCaller.set_fuel(store, 10)\n    iex> Wasmex.StoreOrCaller.get_fuel(store)\n    {:ok, 10}","title":"Examples - Wasmex.StoreOrCaller.get_fuel/1","ref":"Wasmex.StoreOrCaller.html#get_fuel/1-examples"},{"type":"function","doc":"Sets fuel to for Wasm to consume while executing.\n\nFor this method to work, fuel consumption must be enabled via\n`Wasmex.EngineConfig.consume_fuel/2`. By default a `Wasmex.Store`\nstarts with 0 fuel for Wasm to execute with (meaning it will\nimmediately trap and halt execution). This function must be\ncalled for the store to have some fuel to allow WebAssembly\nto execute.\n\nMost Wasm instructions consume 1 unit of fuel. Some\ninstructions, such as `nop`, `drop`, `block`, and `loop`, consume 0\nunits, as any execution cost associated with them involves other\ninstructions which do consume fuel.\n\nNote that at this time when fuel is entirely consumed it will cause\nWasm to trap.","title":"Wasmex.StoreOrCaller.set_fuel/2","ref":"Wasmex.StoreOrCaller.html#set_fuel/2"},{"type":"function","doc":"This function will return an error if fuel consumption is not enabled\nvia `Wasmex.EngineConfig.consume_fuel/2`.","title":"Errors - Wasmex.StoreOrCaller.set_fuel/2","ref":"Wasmex.StoreOrCaller.html#set_fuel/2-errors"},{"type":"function","doc":"iex> {:ok, engine} = Wasmex.Engine.new(%Wasmex.EngineConfig{consume_fuel: true})\n    iex> {:ok, store} = Wasmex.Store.new(nil, engine)\n    iex> Wasmex.StoreOrCaller.set_fuel(store, 10)\n    :ok","title":"Examples - Wasmex.StoreOrCaller.set_fuel/2","ref":"Wasmex.StoreOrCaller.html#set_fuel/2-examples"},{"type":"type","doc":"","title":"Wasmex.StoreOrCaller.t/0","ref":"Wasmex.StoreOrCaller.html#t:t/0"},{"type":"module","doc":"Options for preopening a directory.\n\nLikely to be extended with read/write permissions once wasmtime supports them.","title":"Wasmex.Wasi.PreopenOptions","ref":"Wasmex.Wasi.PreopenOptions.html"},{"type":"module","doc":"* `:path` - The path to the directory to preopen\n  * `:alias` - The alias to use for the directory. The directory will be\n    available at this path in the WASI filesystem. If not specified, the\n    directory will be available at its real path.","title":"Options - Wasmex.Wasi.PreopenOptions","ref":"Wasmex.Wasi.PreopenOptions.html#module-options"},{"type":"module","doc":"iex> Wasmex.Store.new_wasi(%Wasmex.Wasi.WasiOptions{\n      ...>   preopen: [\n      ...>     %PreopenOptions{path: \"/tmp\", alias: \"temp\"}\n      ...>   ],\n      ...> })","title":"Example - Wasmex.Wasi.PreopenOptions","ref":"Wasmex.Wasi.PreopenOptions.html#module-example"},{"type":"type","doc":"","title":"Wasmex.Wasi.PreopenOptions.t/0","ref":"Wasmex.Wasi.PreopenOptions.html#t:t/0"},{"type":"module","doc":"Configures WASI support for a Wasmex.Store.","title":"Wasmex.Wasi.WasiOptions","ref":"Wasmex.Wasi.WasiOptions.html"},{"type":"module","doc":"* `:args` - A list of command line arguments\n  * `:env` - A map of environment variables\n  * `:preopen` - A list of `Wasmex.Wasi.PreopenOptions` to preopen directories\n  * `:stdin` - A `Wasmex.Pipe` to use as stdin\n  * `:stdout` - A `Wasmex.Pipe` to use as stdout\n  * `:stderr` - A `Wasmex.Pipe` to use as stderr","title":"Options - Wasmex.Wasi.WasiOptions","ref":"Wasmex.Wasi.WasiOptions.html#module-options"},{"type":"module","doc":"iex> {:ok, stdin} = Wasmex.Pipe.new()\n    iex> Wasmex.Store.new_wasi(%WasiOptions{\n    ...>   args: [\"first param\", \"second param\"],\n    ...>   env: %{\"env_key\" => \"env_value\"},\n    ...>   preopen: [\n    ...>     %Wasmex.Wasi.PreopenOptions{path: \"/tmp\"}\n    ...>   ],\n    ...>   stdin: stdin,\n    ...> })","title":"Example - Wasmex.Wasi.WasiOptions","ref":"Wasmex.Wasi.WasiOptions.html#module-example"},{"type":"type","doc":"","title":"Wasmex.Wasi.WasiOptions.t/0","ref":"Wasmex.Wasi.WasiOptions.html#t:t/0"},{"type":"module","doc":"Configures WASI P2 support for a Wasmex.Components.Store.\n\nWASI (WebAssembly System Interface) P2 provides system interface capabilities\nto WebAssembly components, allowing them to interact with the host system in a\ncontrolled manner.","title":"Wasmex.Wasi.WasiP2Options","ref":"Wasmex.Wasi.WasiP2Options.html"},{"type":"module","doc":"* `:inherit_stdin` - When `true`, allows the component to read from standard input.\n    Defaults to `true`.\n\n  * `:inherit_stdout` - When `true`, allows the component to write to standard output.\n    Defaults to `true`.\n\n  * `:inherit_stderr` - When `true`, allows the component to write to standard error.\n    Defaults to `true`.\n\n  * `:allow_http` - When `true`, enables HTTP capabilities for the component.\n    Defaults to `false`.\n\n  * `:args` - List of command-line arguments to pass to the component.\n    Defaults to `[]`.\n\n  * `:env` - Map of environment variables to make available to the component.\n    Defaults to `%{}`.","title":"Options - Wasmex.Wasi.WasiP2Options","ref":"Wasmex.Wasi.WasiP2Options.html#module-options"},{"type":"module","doc":"iex> wasi_opts = %Wasmex.Wasi.WasiP2Options{\n    ...>   args: [\"--verbose\"],\n    ...>   env: %{\"DEBUG\" => \"1\"},\n    ...>   allow_http: true\n    ...> }\n    iex> {:ok, pid} = Wasmex.Components.start_link(%{\n    ...>   path: \"my_component.wasm\",\n    ...>   wasi: wasi_opts\n    ...> })","title":"Example - Wasmex.Wasi.WasiP2Options","ref":"Wasmex.Wasi.WasiP2Options.html#module-example"},{"type":"type","doc":"","title":"Wasmex.Wasi.WasiP2Options.t/0","ref":"Wasmex.Wasi.WasiP2Options.html#t:t/0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}